#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Server_1 = require("./Server");
const services = require("./services");
const program = require("commander");
const path = require("path");
class Cli {
    constructor() {
        this.program = new program.Command();
        this.serverConfig = {
            dataPath: path.resolve("data")
        };
    }
    start(argv) {
        this.program.version(require('../package.json').version);
        this.program
            .command('start')
            .description('start Realm Object Server')
            .option("-d, --data <path>", "Specify the data path to use", "./data")
            .option("-a, --address <address>", "Specify the address to listen on", "0.0.0.0")
            .option("-p, --port <port>", "Specify the port to listen on", 9080)
            .action((command) => {
            const config = Object.assign(this.serverConfig, {
                dataPath: path.resolve(command.data),
                hostname: command.address,
                port: command.port,
            });
            const server = new Server_1.Server(config);
            server.addService(new services.UsernamePasswordAuthService({
                privateKeyPath: path.join(config.dataPath, "keys", "auth.key"),
            }));
            server.addService(new services.RealmManagementService({
                privateKeyPath: path.join(config.dataPath, "keys", "auth.key"),
            }));
            server.addService(new services.LogService());
            server.addService(new services.SyncProxyService());
            server.addService(new services.SyncService({
                dataPath: path.resolve(config.dataPath, "sync"),
                publicKeyPath: path.resolve(config.dataPath, "keys", "auth.pub"),
            }));
            process.removeAllListeners("SIGINT");
            process.on('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
                console.log("Got signal SIGINT.  Shutting down.");
                yield server.shutdown();
                process.exit(0);
            }));
            process.removeAllListeners("SIGTERM");
            process.on('SIGTERM', () => __awaiter(this, void 0, void 0, function* () {
                console.log("Got signal SIGTERM.  Shutting down.");
                yield server.shutdown();
                process.exit(0);
            }));
            server.start().catch(err => console.error(err));
        });
        this.program
            .command('init')
            .description('Initialize a new ROS project')
            .action(() => {
            console.error("init: not yet implemented");
            process.exit(1);
        });
        if (!argv.slice(2).length) {
            this.program.outputHelp();
            process.exit(1);
        }
        return this.program.parse(argv);
    }
}
exports.Cli = Cli;
if (!module.parent) {
    new Cli().start(process.argv);
}
//# sourceMappingURL=cli.js.map