"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require("./");
class SingleProcessDiscovery extends _1.Discovery {
    constructor() {
        super(...arguments);
        this.serviceHandles = [];
    }
    registerService(service, serverAddress, serverPort) {
        return __awaiter(this, void 0, void 0, function* () {
            const baseRoute = Reflect.get(service.constructor, 'baseRoute');
            const addressFunctionName = Reflect.get(service, 'addressFunctionName');
            const portFunctionName = Reflect.get(service, 'portFunctionName');
            const addressPropertyKey = Reflect.get(service, 'addressPropertyKey');
            const portPropertyKey = Reflect.get(service, 'portPropertyKey');
            let address = serverAddress;
            if (addressFunctionName) {
                address = yield Promise.resolve(service[addressFunctionName]());
            }
            if (addressPropertyKey) {
                address = service[addressPropertyKey];
            }
            let port = serverPort;
            if (addressFunctionName) {
                port = yield Promise.resolve(service[portFunctionName]());
            }
            if (addressPropertyKey) {
                port = service[portPropertyKey];
            }
            const serviceHandle = {
                name: Reflect.get(service.constructor, 'serviceName') || service.constructor.name,
                address: address,
                port: port,
                tags: service.tags || [],
            };
            this.serviceHandles.push(serviceHandle);
        });
    }
    deregisterService(service) {
        return __awaiter(this, void 0, void 0, function* () {
            const serviceName = (Reflect.get(service.constructor, 'serviceName') || service.constructor.name);
            const serviceHandle = this.serviceHandles.find(s => s.name === serviceName);
            if (serviceHandle) {
                let index = this.serviceHandles.indexOf(serviceHandle);
                if (index >= 0) {
                    this.serviceHandles.splice(index, 1);
                }
            }
        });
    }
    findAll(name, tags) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.serviceHandles.filter((handle) => {
                return handle.name === name;
            }).filter((handle) => {
                if (tags) {
                    let found = false;
                    for (const t of tags) {
                        if (handle.tags.find(t2 => t2 === t)) {
                            found = true;
                        }
                    }
                    return found;
                }
                else {
                    return true;
                }
            });
        });
    }
}
exports.SingleProcessDiscovery = SingleProcessDiscovery;
//# sourceMappingURL=SingleProcessDiscovery.js.map