/// <reference types="express" />
import * as Realm from 'realm';
import { Request } from 'express';
import { Discovery } from '../discovery/Discovery';
export interface AuthenticationResponseJSON {
    refresh_token: {
        token: string;
        token_data: {
            identity: string;
            is_admin: boolean;
            app_id?: string;
            access: ["refresh"];
            expires: number;
        };
    };
    authToken?: string;
}
export interface ResolvedUser {
    userId: string;
    isAdmin: boolean;
    isCreated: boolean;
}
export declare type UserMetadata = {
    [key: string]: string;
};
export declare type User = {
    userId: string;
    isAdmin: boolean;
    metadata?: UserMetadata;
};
export interface AuthServiceConfiguration {
    privateKeyPath?: string;
}
export declare class AuthService {
    privateKey: string;
    realm: Realm;
    discovery: Discovery;
    config: AuthServiceConfiguration;
    constructor(config?: AuthServiceConfiguration);
    authenticateOrCreateUser(body: any): Promise<ResolvedUser>;
    discoveryReady(discovery: Discovery): Promise<void>;
    authenticate(request: Request): Promise<AuthenticationResponseJSON>;
    getUser(userId: string): User;
    getUserMetadata(userId: string): UserMetadata;
    storeUser(userId: string, isAdmin: boolean, metadata?: {
        [key: string]: string;
    }): User;
}
