/// <reference types="express" />
import { Discovery } from '../discovery';
import { Logger } from '../shared/Logger';
import * as Realm from 'realm';
import { Request } from 'express';
export interface RealmAccessTokenResponse {
    access_token: {
        token: string;
        token_data: any;
    };
}
export interface RealmManagementServiceConfig {
    privateKeyPath: string;
    tags?: string[];
}
export declare class RealmManagementService {
    logger: Logger;
    privateKey: string;
    realm: Realm;
    tags: string[];
    config: RealmManagementServiceConfig;
    private discovery;
    constructor(config: RealmManagementServiceConfig);
    private accessLevelForPath(path, userId);
    setLogger(l: Logger): void;
    accessToken(path: string, data: any, app_id?: string): Promise<RealmAccessTokenResponse>;
    find(): Promise<{
        path: string;
        creatorId: string;
        creationDate: Date;
        permissions: Realm.List<{
            path: string;
            accessLevel: number;
            type: number;
            key: string;
            value: string;
        }>;
    }[]>;
    findByPath(realmPath: string, accessToken: string): Promise<{
        path: string;
        creatorId: string;
        creationDate: Date;
        permissions: Realm.List<{
            path: string;
            accessLevel: number;
            type: number;
            key: string;
            value: string;
        }>;
    }>;
    private validatedRealmFile(path);
    private realmFileJSON(realmFile, withPermissions);
    get(incomingRequest: Request): Promise<{
        path: string;
        creatorId: string;
        creationDate: Date;
        permissions?: {
            key: string;
            value: string;
            type: "user" | "all" | "metadata_exact" | "metadata_regex";
            accessLevel: "none" | "read" | "write" | "admin";
        }[];
    }>;
    remove(incomingRequest: Request): Promise<void>;
    getPermissions(incomingRequest: Request): Promise<{
        key: string;
        value: string;
        type: "user" | "all" | "metadata_exact" | "metadata_regex";
        accessLevel: "none" | "read" | "write" | "admin";
    }[]>;
    createPermissions(incomingRequest: Request): Promise<{}>;
    discoveryReady(discovery: Discovery): Promise<void>;
    private createRealmFile(path, creatorId);
}
