"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorators_1 = require("../decorators");
const shared_1 = require("../shared");
const Discovery_1 = require("../discovery/Discovery");
const Logger_1 = require("../shared/Logger");
const httpProxy = require("http-proxy");
const superagent = require("superagent");
let SyncProxyService = class SyncProxyService {
    discoveryReady(discovery) {
        return __awaiter(this, void 0, void 0, function* () {
            this.discovery = discovery;
        });
    }
    setLogger(logger) {
        this.logger = logger;
    }
    websocketHandler(req, socket, head) {
        return __awaiter(this, void 0, void 0, function* () {
            const token = req.headers['x-realm-access-token'];
            if (!token) {
                throw new shared_1.JSONError({
                    title: "X-Realm-Access-Token header was not provided",
                    status: 400,
                });
            }
            let parsedToken;
            try {
                parsedToken = shared_1.parseTokenFromBase64RSA_SHA256(token);
            }
            catch (err) {
            }
            if (!parsedToken) {
                throw new shared_1.JSONError({
                    title: "X-Realm-Access-Token header is not valid",
                    status: 400,
                });
            }
            let realmFilePath = req.headers['x-realm-path'];
            if (!realmFilePath) {
                throw new shared_1.JSONError({
                    title: "X-Realm-Path header is not valid",
                    status: 400,
                });
            }
            realmFilePath = encodeURIComponent(realmFilePath);
            let syncLabel = parsedToken.syncLabel;
            if (!syncLabel || syncLabel === '') {
                const realmsServiceHandle = yield this.discovery.find('realms');
                const realmsServiceUrl = new URI(`http://${realmsServiceHandle.address}:${realmsServiceHandle.port}`)
                    .segment("/realms/files/")
                    .segment(`/${realmFilePath}`)
                    .toString();
                try {
                    const response = yield superagent.get(realmsServiceUrl)
                        .set({
                        'X-Realm-Access-Token': token
                    });
                    syncLabel = response.body.syncLabel;
                }
                catch (err) {
                    throw new shared_1.JSONError({
                        title: "Unable find the realm file",
                        status: 503,
                    });
                }
            }
            const label = parsedToken.syncLabel;
            if (!label || label === "") {
                throw new shared_1.JSONError({
                    title: "X-Realm-Access-Token header is not valid: sync label was not found in token",
                    status: 400,
                });
            }
            const syncHandle = yield this.discovery.find("sync", ["master", label]);
            if (!syncHandle) {
                throw new shared_1.JSONError({
                    title: "Backend sync service is unavailable",
                    status: 503,
                });
            }
            const target = `http://${syncHandle.address}:${syncHandle.port}`;
            const proxy = httpProxy.createProxy({ ws: true, target });
            proxy.on('error', (err) => {
                this.logger.error("sync proxy error", err);
            });
            proxy.on('close', () => {
                this.logger.error("sync proxy closed");
            });
            proxy.ws(req, socket, head);
        });
    }
};
__decorate([
    decorators_1.InjectDiscovery,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Discovery_1.Discovery]),
    __metadata("design:returntype", Promise)
], SyncProxyService.prototype, "discoveryReady", null);
__decorate([
    decorators_1.Unmute(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Logger_1.Logger]),
    __metadata("design:returntype", void 0)
], SyncProxyService.prototype, "setLogger", null);
__decorate([
    decorators_1.Upgrade('/'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], SyncProxyService.prototype, "websocketHandler", null);
SyncProxyService = __decorate([
    decorators_1.BaseRoute('/realm-object-server')
], SyncProxyService);
exports.SyncProxyService = SyncProxyService;
//# sourceMappingURL=SyncProxyService.js.map